@@ -20,6 +20,8 @@
  * Boston, MA  02110-1301  USA
  */
 
+#include <libxml2/libxml/tree.h>
+
 #include "driver.h"
 #include "vkontakte.h"
 #include "parser.h"
@@ -90,7 +92,9 @@ xmlChar* event_message = xmlCharStrdup("\
     doc_root = xmlParseDoc(event_struct);
     node = xmlDocGetRootElement(doc);
     xmlUnlinkNode(node);
-    xmlDocSetRootElement(doc, NULL);
+    xmlNodePtr oldroot = xmlDocSetRootElement(doc, NULL);
+    if (oldroot != NULL)
+        xmlFreeNode(oldroot);
     
     xmlXPathObject* Obj = xpath("//TransitData/Content/Signal/Content", doc_root);
     node_root = Obj->nodesetval->nodeTab[0];
@@ -204,7 +208,9 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     xmlNodePtr rootNode = NULL, node = NULL, childNode = NULL;
     xmlDocPtr reqXml2 = NULL;
     rootNode = xmlNewDocNode(doc, NULL, BAD_CAST "Response", NULL);
-    xmlDocSetRootElement(doc, rootNode);
+    xmlNodePtr oldRoot = xmlDocSetRootElement(doc, rootNode);
+    if (oldRoot != NULL)
+        xmlFreeNode(oldRoot);
 
     class_val = get_req_class(reqXml);
     function = get_req_function(reqXml);
@@ -255,7 +261,9 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
             childNode = xmlCopyNode(node,1);
             xmlUnlinkNode(childNode);
             reqXml2 = xmlNewDoc(BAD_CAST "1.0");
-            xmlDocSetRootElement(reqXml2, childNode);
+            xmlNodePtr oldChild = xmlDocSetRootElement(reqXml2, childNode);
+            if (oldChild != NULL)
+                xmlFreeNode(oldChild);
             xmlXPathFreeObject(Obj);
         } else {
             return 1;
@@ -346,6 +354,7 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
         if (g_strcmp0((gchar *)function, GET_BINARY_DATA) == 0 ) {
             g_free(function);
             if (counter == -1) {
+        	g_free(url);
                 return 0;
             }
             parser_image(doc, tmp, url);
@@ -494,10 +503,12 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
 
     } else if (code >= 500){
         info_message(DRV_SERVER_ERROR_CODE, doc, DRV_SERVER_ERROR, 1);
+        g_free(url);
         g_free(function);
         return 0;
     } else if (code == 400){
         info_message(DRV_NETW_ERROR_CODE, doc, DRV_NETW_ERROR, 1);
+        g_free(url);
         g_free(function);
         return 0;
     }
@@ -747,36 +758,40 @@ void unset_settings(char* name){
  */
 int authorize(char *email, char *pass)
 {
-    char *body;
+    char *body = NULL;
     char* mid = NULL, sid[MID_SIZE];
     char *tmp;
     gchar* post_data;
     int code = 0;
-    struct curl_slist *cookies;
+    struct curl_slist *cookies = NULL;
     gchar* path;
 
     GConfClient* client;
     client = gconf_client_get_default ();
-
-    g_print("authorize START");
     if ( email == NULL || pass == NULL) {
         g_debug("email = NULL");
         return 1;
     }
-    post_data = g_strdup_printf("email=%s&pass=%s&expire=0&vk=", email, pass);
 
 #ifdef TEST
     info_proxy = CTF_TEST_DATA("PROXY");
     info_port = atoi(CTF_TEST_DATA("PORT"));
 #endif
 
-    code = http_post("http://login.vk.com/?act=login", info_proxy, info_port, post_data, NULL, &body, &cookies);
+    g_debug("%s:%d:%s(): Get mid", __FILE__, __LINE__, __FUNCTION__);
+    post_data = g_strdup_printf("email=%s&pass=%s&expire=&vk=1&try_to_login=1", email, pass);
+
+    code = http_post("http://login.vk.com/?act=login", 
+		     info_proxy, info_port, post_data, NULL, &body, &cookies);
     g_free(post_data);
     if (code == 200) {
 
         int i = 0;
 	gchar** cookie;
+
+        struct curl_slist *cookies_orig = cookies;
 	while (cookies) {
+		g_debug("%s:%d:%s: cookie=%s", __FILE__, __LINE__, __FUNCTION__, cookies->data);
 	    cookie = g_strsplit(cookies->data,"\t", 20);
 	    i = 0;
 	    while(cookie[i] != NULL){
@@ -789,7 +804,8 @@ int authorize(char *email, char *pass)
             g_strfreev(cookie);
             cookies = cookies->next;
         }
-        curl_slist_free_all(cookies);
+
+        curl_slist_free_all(cookies_orig);
 	if ( mid != NULL ) {
 	    path = g_strconcat(PATH, REMIXMID, NULL);
 	    gconf_client_set_string (client, path, mid, NULL);
@@ -805,7 +821,11 @@ int authorize(char *email, char *pass)
     //Get remixsid
         g_debug("BODY %s", body);
         if(body != NULL){
-            tmp = g_strrstr(body, "id='s' value='") + 14;
+    	    tmp = g_strrstr(body, "name='s' value='");
+    	    if (tmp != NULL) {
+        	tmp += strlen("name='s' value='");
+            }
+            
             if(tmp == NULL){
                 path = g_strconcat(PATH, REMIXMID, NULL);
                 gconf_client_unset(client, path, NULL);
@@ -828,11 +848,13 @@ int authorize(char *email, char *pass)
 	gconf_client_unset(client, path, NULL);
 	g_free(path);
         return 1;
-    } else 
-          if (code == 400)
-              return 2;
-          else    
-              return 1;
+    } else {
+        curl_slist_free_all(cookies);
+        if (code == 400)
+            return 2;
+        else    
+            return 1;
+    }
 }
 
 
@@ -1031,8 +1053,8 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
         return NULL;
 
     url = xmlNodeGetContent(root);
-    
-    xsltFreeStylesheet(cur);    
+
+    xsltFreeStylesheet(cur);
     xmlFreeDoc(urlXml);
     g_free((gpointer*)params[0]);
     g_free((gpointer*)params[1]);
@@ -1065,7 +1087,9 @@ int info_message(char* code, xmlDocPtr doc, char* message, int type)
 {
     xmlNodePtr rootNode, node, childNode;
     rootNode = xmlNewDocNode(doc, NULL, BAD_CAST "Response", NULL);
-    xmlDocSetRootElement(doc, rootNode);
+    xmlNodePtr oldroot = xmlDocSetRootElement(doc, rootNode);
+    if (oldroot != NULL)
+        xmlFreeNode(oldroot);
 
     xmlSetProp(rootNode, BAD_CAST CLASS, BAD_CAST "systemMessages");
 
@@ -1255,7 +1279,7 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
 
 
     c = curl_easy_init();
-    g_print("url in http_post %s", url);
+    g_debug("url in http_post %s", url);
 
     curl_easy_setopt(c, CURLOPT_URL, url);
 
@@ -1276,7 +1300,7 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
 
     curl_easy_setopt(c, CURLOPT_WRITEDATA, (void *)&chunk);
     curl_easy_setopt(c, CURLOPT_WRITEHEADER, (void *)&chunk2);
-    g_debug("send request");
+    g_debug("%s:%d:%s():send request", __FILE__, __LINE__, __FUNCTION__);
     
     if (curl_easy_perform(c) != 0) {
         g_warning("Can't perform the request\n");
@@ -1287,18 +1311,21 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
     res = curl_easy_getinfo(c, CURLINFO_COOKIELIST, &*cookies);
     if (res != CURLE_OK) {
         curl_easy_cleanup(c);
+        curl_slist_free_all(*cookies);
+        *cookies = NULL;
         return 400;
     }
     
     curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
     g_debug("CODE %d", code);
+    g_debug("HEADER\n  %s", chunk2.memory);
+    //g_debug("BODY====\n  %s\n=====BODY_END======", chunk.memory);
     
     
     if (header != NULL) {
         *header = g_strdup(chunk2.memory);
     }
     
-    g_print("HEADER  %s\n", chunk2.memory);
     if(chunk2.memory)
         g_free(chunk2.memory);
     
